@@ -1,9 +1,10 @@
-#!env perl
+#!/usr/bin/perl
 use strict;
 use warnings FATAL => 'all';
 use diagnostics;
 use MarpaX::Languages::C::AST;
 use Getopt::Long;
+use Pod::Usage;
 use IPC::Run qw/run/;
 use Term::ProgressBar;
 use POSIX qw/EXIT_FAILURE EXIT_SUCCESS/;
@@ -15,16 +16,16 @@ use Data::Dumper;
 use Log::Log4perl qw/:easy/;
 use Log::Any::Adapter;
 use Log::Any qw/$log/;
+use File::Spec;
 
 autoflush STDOUT 1;
 
 # ABSTRACT: C source analysis
 
-our $VERSION = '0.20'; # TRIAL VERSION
+our $VERSION = '0.27'; # TRIAL VERSION
 
 # PODNAME: c2ast.pl
 
-
 my $help = 0;
 my @cpp = ();
 my $cppfile = '';
@@ -65,8 +66,11 @@ DEFAULT_LOG4PERL_CONF
 Log::Log4perl::init(\$defaultLog4perlConf);
 Log::Any::Adapter->set('Log4perl');
 
-if ($help || ! @ARGV) {
-  usage($help ? EXIT_SUCCESS : EXIT_FAILURE)
+if ($help) {
+    pod2usage(-exitval => EXIT_SUCCESS);
+}
+if (! @ARGV) {
+    pod2usage(-exitval => EXIT_FAILURE)
 }
 
 my $preprocessedOutput;
@@ -91,7 +95,7 @@ if ($cppdup) {
 # -----------------
 # Callback argument
 # -----------------
-my %lexemeCallbackHash = (file => $cppfile,
+my %lexemeCallbackHash = (file => File::Spec->canonpath($cppfile),
 			  lexeme => {},
 			  internalLexeme => {},
 			  progress => undef,
@@ -266,10 +270,10 @@ sub lexemeCallback {
 	if ($lexemeHashp->{value} =~ /([\d]+)\s*\"([^\"]+)\"/) {
 	    $lexemeCallbackHashp->{curline} = substr($lexemeHashp->{value}, $-[1], $+[1] - $-[1]);
 	    $lexemeCallbackHashp->{curline_real} = $lexemeHashp->{line};
-	    $lexemeCallbackHashp->{curfile} = substr($lexemeHashp->{value}, $-[2], $+[2] - $-[2]);
+	    $lexemeCallbackHashp->{curfile} = File::Spec->canonpath(substr($lexemeHashp->{value}, $-[2], $+[2] - $-[2]));
 	    $lexemeCallbackHashp->{allfiles}->{$lexemeCallbackHashp->{curfile}}++;
 	    if (! $lexemeCallbackHashp->{file}) {
-		$lexemeCallbackHashp->{file} = $lexemeCallbackHashp->{curfile};
+		$lexemeCallbackHashp->{file} = File::Spec->canonpath($lexemeCallbackHashp->{curfile});
 	    }
 	    if (! defined($lexemeCallbackHashp->{tryToAlignMax})) {
 		$lexemeCallbackHashp->{tryToAlignMax} = length(sprintf('%s(%d)', $lexemeCallbackHashp->{file}, 1000000)); # a pretty good max -;
@@ -293,118 +297,13 @@ sub lexemeCallback {
 	    my $line = $lexemeCallbackHashp->{curline} + ($lexemeHashp->{line} - $lexemeCallbackHashp->{curline_real} - 1);
 	    $lexemeCallbackHashp->{position2line}->{$lexemeHashp->{start}} = $line;
 	    if (exists($lexemeCallbackHashp->{lexeme}->{$lexemeHashp->{name}})) {
-		my $tryToAlign = sprintf('%s(%d)', $lexemeCallbackHashp->{curfile}, $line);
+		my $tryToAlign = sprintf('%s(%d)', $lexemeCallbackHashp->{file}, $line);
 		printf "%-*s %-30s %s\n", $lexemeCallbackHashp->{tryToAlignMax}, $tryToAlign, $lexemeHashp->{name}, $lexemeHashp->{value};
 	    }
 	}
     }
 
 }
-# --------------------------------------------------------------------------------------
-sub usage {
-    my $rc = shift;
-
-    print <<USAGE;
-Usage: $^X $0 options
-
-where options can be:
-
---help               This help
-
---cpp <argument>     cpp executable. Default is 'cpp'.
-
-                     If your setup requires additional option, then you should repeat this option.
-                     For example: your cpp setup is "cl -E". Then you say: --cpp cl --cpp -E
-
-                     Take care: it has been observed that "cpp" output could be different than "compiler -E".
-                     If c2ast complains and the output manifestly reports something that has not been
-                     preprocessed corrected, then retry with: --cpp your_compiler --cpp your_compiler_option
-
-                     This has been observed on Darwin for instance, where one have to say:
-                     --cpp gcc --cpp -E
-
---cppfile <filename> The output of the preprocessor will contain a lot of lines like '#line "filename"', where
-                     "filename" is guaranteed to be the constant for a given physical location in the filesystem.
-                     The first occurence is always a marker of the source that c2ast sent to the preprocessor.
-                     c2ast will catch it so that the --lexeme tracing phase, or the --check phase, will happen only
-                     on lexemes relevant to the source file given on the command-line, and not polluted by lexemes
-                     coming from any included file.
-
-                     Nevertheless, the source given on the command-line /could/ be the result of another preprocessing phase.
-                     You can use this option to tell c2ast what is the real filename to consider.
-
-                     For example, suppose your .c file has the name: my/generated/source.c. And that the real origin is
-                     my/input/source.w, where my/input/source.w has been preprocessed by you to generate
-                     my/generated/source.c.
-
-                     Then, it is very likely that my/generated/source.c will contain a line like:
-                     # line xxx "my/input/source.c"
-
-                     You will then say --cppfile "my/input/source.c".
-
-                     Exemple: the file marpa.c in the build phase of libmarpa: this is the result of
-                     a preprocessing on the file marpa.w.
-
---cppdup <filename>  Save the preprocessed output to this filename. Only useful for debugging c2ast.
-
---lexeme <lexeme>    Lexemes of interest. Look to the grammar to have the exhaustive list.
-                     In practice, only IDENTIFIER, TYPEDEF_NAME and ENUMERATION_CONSTANT are useful.
-                     An internal lexeme, not generated by Marpa itself also exist: PREPROCESSOR_LINE_DIRECTIVE.
-                     This option must be repeated for every lexeme of interest.
-                     Giving a value __ALL__ will make all lexemes candidates for logging.
-                     The output will go to STDOUT.
-
---progress           Progress bar with ETA information. The "name" associated with the progress bar will the last
-                     of the arguments unknown to c2ast. So it is quite strongly suggested to always end your
-                     command-line with the file you want to analyse.
-
---check <checkName>  Perform some hardcoded checks on the code. Supported values for checkName are:
-  reservedNames      Check IDENTIFIER lexemes v.s. Gnu recommended list of Reserved Names [1].
-
-                     Any check that is not ok will print on STDERR.
-
---dump               Dump parse tree value on STDOUT.
---dumpfile <file>    Dump parse tree value to this named file.
-
-                     Take care: dumping the parse tree value can hog your memory and CPU. This will not
-                     be c2ast fault, but the module used to do the dump (currently, Data::Dumper).
-
---allowAmbiguity     Default is to allow a single parse tree value. Nevertheless, if the grammar in use by
-                     c2ast has a hole, use this option to allow multiple parse tree values. In case of multiple
-                     parse tree values, only the first one will be used in the check phase (option --check).
-
---loglevel <level>   A level that has to be meaningful for Log::Log4perl, typically WARN, INFO, ERROR, etc.
-                     Default is WARN.
-
-                     Note that tracing Marpa library itself is possible, but only using environment variable MARPA_TRACE /and/ saying --loglevel TRACE.
-
-                     In case of trouble, typical debuggin of c2ast is:
-                     --loglevel INFO
-                     then:
-                     --loglevel DEBUG
-                     then:
-                     --loglevel TRACE
-
---logstderr          Logs to stderr or not. Default is $logstderr.
-
-Examples:
-
-$0                   -D MYDEFINE1 -D MYDEFINE2 -I       /tmp/myIncludeDir            /tmp/myfile.c
-$0                   -D MYDEFINE1 -D MYDEFINE2 -I       /tmp/myIncludeDir            /tmp/myfile.c --lexeme IDENTIFIER --lexeme TYPEDEF_NAME
-$0 --cpp cl --cpp -E -D MYDEFINE1 -D MYDEFINE2 -I C:/Windows/myIncludeDir C:/Windows/Temp/myfile.c
-$0                   -D MYDEFINE1 -D MYDEFINE2 -I       /tmp/myIncludeDir            /tmp/myfile.c --progress --check reservedNames
-
-Less typical usage:
-
-$0 -I libmarpa_build --cpp gcc --cpp -E --cppfile ./marpa.w  --progress --check reservedNames libmarpa_build/marpa.c
-
-[1] http://www.gnu.org/software/libc/manual/html_node/Reserved-Names.html
-USAGE
-
-    exit($rc);
-}
-
-__END__
 
 =pod
 
@@ -416,7 +315,25 @@ c2ast.pl - C source analysis
 
 =head1 VERSION
 
-version 0.20
+version 0.27
+
+=head1 SYNOPSIS
+
+c2ast.pl [options] [file ...]
+
+ Options:
+   --help               Brief help message
+   --cpp <argument>     cpp executable. Default is 'cpp'.
+   --cppfile <filename> The name of the file being preprocessed.
+   --cppdup <filename>  Save the preprocessed output to this filename.
+   --lexeme <lexeme>    Lexemes of interest.
+   --progress           Progress bar with ETA information.
+   --check <checkName>  Perform some hardcoded checks on the code.
+   --dump               Dump parse tree value on STDOUT.
+   --dumpfile <file>    Dump parse tree value to this named file.
+   --allowAmbiguity     Allow more than a single parse tree value.
+   --loglevel <level>   A level that has to be meaningful for Log::Log4perl, typically WARN, INFO, ERROR, etc.
+   --logstderr          Logs to stderr or not.
 
 =head1 DESCRIPTION
 
@@ -434,6 +351,145 @@ This script will use Marpa::R2 to analyse the file given in argument.
 
 Say --help on the command-line to have the full list of options, and examples.
 
+=head1 NAME
+
+c2ast.pl - C source code transformation to AST and eventual check of C Programming Best Practices
+
+=head1 OPTIONS
+
+=over 8
+
+=item B<--help>
+
+This help
+
+=item B<--cpp argument>
+
+cpp executable. Default is 'cpp'.
+
+If your setup requires additional option, then you should repeat this option.
+For example: your cpp setup is "cl -E". Then you say: --cpp cl --cpp -E
+
+Take care: it has been observed that "cpp" output could be different than "compiler -E".
+If c2ast complains and the output manifestly reports something that has not been
+preprocessed corrected, then retry with: --cpp your_compiler --cpp your_compiler_option
+
+This has been observed on Darwin for instance, where one have to say:
+--cpp gcc --cpp -E
+
+=item B<--cppfile filename>
+
+The name of the file being preprocessed.  Usually this option is not necessary.  By
+default this is the main file being pre-processed, as indicated by the first preprocessor
+line directive.  (Preprocessor line directives start are lines starting with '#line'.)
+For the --lexeme tracing phase or the --check phase, c2ast includes only the information
+that is relevant to the lexemes contained in the "cppfile".
+
+One circumstance where this option is necessary if when the C files are not the source
+files, but were generated by another program.  For example, the input file might be named
+'generated.c', but the actual source, from which 'generated.c' was generated, might be
+in a file named 'source.c'.  If convention is being followed, 'generated.c' will contain
+lines of the form, to indicate which portions of the code originally came from 'source.c'.
+
+  # line xxx "source.c"
+
+You can tell c2ast.pl to analyze the code originally from "source.c", as indicated by the
+preprocessor line directives, with the option
+
+--cppfile "source.c"
+
+=item B<--cppdup filename>
+
+Save the preprocessed output to this filename. Only useful for debugging c2ast.
+
+=item B<--lexeme lexeme>
+
+Lexemes of interest. Look to the grammar to have the exhaustive list.
+In practice, only IDENTIFIER, TYPEDEF_NAME and ENUMERATION_CONSTANT are useful.
+An internal lexeme, not generated by Marpa itself also exist: PREPROCESSOR_LINE_DIRECTIVE.
+This option must be repeated for every lexeme of interest.
+Giving a value __ALL__ will make all lexemes candidates for logging.
+The output will go to STDOUT.
+
+=item B<--progress>
+
+Progress bar with ETA information. The "name" associated with the progress bar will the last
+of the arguments unknown to c2ast. So it is quite strongly suggested to always end your
+command-line with the file you want to analyse.
+
+=item B<--check checkName>
+
+Perform some hardcoded checks on the code. Supported values for checkName are:
+
+=over
+
+=item reservedNames
+
+Check IDENTIFIER lexemes v.s. Gnu recommended list of Reserved Names [1].
+
+=back
+
+Any check that is not ok will print on STDERR.
+
+=item B<--dump>
+
+Dump parse tree value on STDOUT.
+
+=item B<--dumpfile file>
+
+Dump parse tree value to this named file.
+
+Take care: dumping the parse tree value can hog your memory and CPU. This will not
+be c2ast fault, but the module used to do the dump (currently, Data::Dumper).
+
+=item B<--allowAmbiguity>
+
+Default is to allow a single parse tree value. Nevertheless, if the grammar in use by
+c2ast has a hole, use this option to allow multiple parse tree values. In case of multiple
+parse tree values, only the first one will be used in the check phase (option --check).
+
+=item B<--loglevel level>
+
+A level that has to be meaningful for Log::Log4perl, typically WARN, INFO, ERROR, etc.
+Default is WARN.
+
+Note that tracing Marpa library itself is possible, but only using environment variable MARPA_TRACE /and/ saying --loglevel TRACE.
+
+In case of trouble, typical debugging phases c2ast are:
+--loglevel INFO
+then:
+--loglevel DEBUG
+then:
+--loglevel TRACE
+
+=item B<--logstderr>
+
+Logs to stderr or not. Default is $logstderr.
+
+=back
+
+Any option not documented upper will be considered as a cpp option, and sent to the underlying the cpp program.
+
+=head1 EXAMPLES
+
+Examples:
+
+c2ast.pl                   -D MYDEFINE1 -D MYDEFINE2 -I       /tmp/myIncludeDir            /tmp/myfile.c
+
+c2ast.pl                   -D MYDEFINE1 -D MYDEFINE2 -I       /tmp/myIncludeDir            /tmp/myfile.c --lexeme IDENTIFIER --lexeme TYPEDEF_NAME
+
+c2ast.pl --cpp cl --cpp -E -D MYDEFINE1 -D MYDEFINE2 -I C:/Windows/myIncludeDir C:/Windows/Temp/myfile.c
+
+c2ast.pl                   -D MYDEFINE1 -D MYDEFINE2 -I       /tmp/myIncludeDir            /tmp/myfile.c --progress --check reservedNames
+
+Less typical usage:
+
+c2ast.pl -I libmarpa_build --cpp gcc --cpp -E --cppfile ./marpa.w  --progress --check reservedNames libmarpa_build/marpa.c
+
+=head1 SEE ALSO
+
+http://www.gnu.org/software/libc/manual/html_node/Reserved-Names.html
+
 =head1 AUTHOR
 
 Jean-Damien Durand <jeandamiendurand@free.fr>
@@ -446,3 +502,8 @@ This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
 =cut
+
+__END__
+
+# --------------------------------------------------------------------------------------
+
