@@ -13,7 +13,7 @@ use MarpaX::Languages::C::AST::Impl qw//;
 use MarpaX::Languages::C::AST::Scope qw//;
 use MarpaX::Languages::C::AST::Callback::Events qw//;
 
-our $VERSION = '0.20'; # TRIAL VERSION
+our $VERSION = '0.27'; # TRIAL VERSION
 
 
 # ----------------------------------------------------------------------------------------
@@ -232,7 +232,7 @@ sub _doPreprocessing {
     #
     my $previous = pos(${$self->{_sourcep}});
     my $delta = 0;
-    my $line = 0;
+    my $line = 1;
     if ($pos > $[) {
       my $line_columnp = lineAndCol($self->{_impl});
       $line = $line_columnp->[0];
@@ -270,7 +270,7 @@ sub _doPreprocessing {
         #
         $line += ($pre =~ tr/\n//);
 	#
-	# If this is a #line, fake an callback event PREPROCESSOR_LINE_DIRECTIVE
+	# If this is a #line, fake a callback event PREPROCESSOR_LINE_DIRECTIVE
 	#
 	if ($directive eq 'line' || $directive =~ /^\d+$/) {
 	    my %lexeme = ();
@@ -403,7 +403,7 @@ sub _doPauseBeforeLexeme {
 	      $newlexeme = 'IDENTIFIER';
 	  } else {
 	      my $line_columnp = lineAndCol($self->{_impl});
-	      logCroak("[%s] Lexeme value \"%s\" cannot be associated to TYPEDEF_NAME, ENUMERATION_CONSTANT nor IDENTIFIER at line %d, column %d.\n\nLast position:\n\n%s%s", whoami(__PACKAGE__), $lexemeHashp->{value}, $lexemeHashp->{line}, $lexemeHashp->{column}, showLineAndCol(@{$line_columnp}, $self->{_sourcep}), $self->_context());
+	      logCroak("[%s] Lexeme value \"%s\" cannot be associated to TYPEDEF_NAME, ENUMERATION_CONSTANT nor IDENTIFIER at line %d, column %d.\n\nLast position:\n\n%s%s", whoami(__PACKAGE__), $lexemeHashp->{value}, $lexemeHashp->{line}, $lexemeHashp->{column}, showLineAndCol($lexemeHashp->{line}, $lexemeHashp->{column}, $self->{_sourcep}), $self->_context());
 	  }
 	  #
 	  # Push the unambiguated lexeme
@@ -442,7 +442,7 @@ MarpaX::Languages::C::AST - Translate a C source to an AST
 
 =head1 VERSION
 
-version 0.20
+version 0.27
 
 =head1 SYNOPSIS
 
@@ -579,6 +579,10 @@ Jean-Damien Durand <jeandamiendurand@free.fr>
 
 =item *
 
+Jeffrey Kegler <JKEGL@cpan.org>
+
+=item *
+
 Jeffrey Kegler <jkegl@cpan.org>
 
 =item *
