@@ -6,17 +6,17 @@ use MarpaX::Languages::C::AST::Util qw/whoami/;
 
 # ABSTRACT: Scope management when translating a C source to an AST
 
-use Storable qw/dclone/;
 use Log::Any qw/$log/;
 use Carp qw/croak/;
 
-our $VERSION = '0.20'; # TRIAL VERSION
+our $VERSION = '0.27'; # TRIAL VERSION
 
 
 sub new {
   my ($class) = @_;
 
   my $self  = {
+      _nscope => 0,
       _typedefPerScope => [ {} ],
       _enumAnyScope => {},
       _delay => 0,
@@ -48,11 +48,18 @@ sub parseEnterScope {
 
   # $self->condExitScope();
 
-  my $scope = $self->parseScopeLevel;
   if ($log->is_debug) {
-      $log->debugf('[%s] Duplicating scope %d to %d', whoami(__PACKAGE__), $scope, $scope + 1);
+      $log->debugf('[%s] Duplicating scope %d to %d', whoami(__PACKAGE__), $self->{_nscope}, $self->{_nscope} + 1);
   }
-  push(@{$self->{_typedefPerScope}}, dclone($self->{_typedefPerScope}->[$scope]));
+  #
+  # calling Clone::clone is overhead for us:
+  # - user data associated to a typedef is assumed to never be modified: copying the $data itself (i.e. usually a reference) is enough
+  # - We just want to make sure this is a new hash, the values inside the hash can remain identical
+  #
+  # Doing \%{$...} is just to make sure this is a new hash instance, with keys pointing to the same values as the origin
+  #
+  push(@{$self->{_typedefPerScope}}, \%{$self->{_typedefPerScope}->[$self->{_nscope}]});
+  $self->{_nscope}++;
 
   if (@{$self->{_enterScopeCallback}}) {
       my ($ref, @args) = @{$self->{_enterScopeCallback}};
@@ -65,10 +72,9 @@ sub parseEnterScope {
 sub parseDelay {
   my $self = shift;
   if (@_) {
-    my $scope = $self->parseScopeLevel;
     my $value = shift;
     if ($log->is_debug) {
-	$log->debugf('[%s] Setting delay flag to %d at scope %d', whoami(__PACKAGE__), $value, $scope);
+	$log->debugf('[%s] Setting delay flag to %d at scope %d', whoami(__PACKAGE__), $value, $self->{_nscope});
     }
     $self->{_delay} = $value;
   }
@@ -79,7 +85,7 @@ sub parseDelay {
 sub parseScopeLevel {
   my ($self) = @_;
 
-  return $#{$self->{_typedefPerScope}};
+  return $self->{_nscope};
 }
 
 
@@ -112,9 +118,8 @@ sub parseExitScope {
 sub parseReenterScope {
   my ($self) = @_;
 
-  my $scope = $self->parseScopeLevel;
   if ($log->is_debug) {
-      $log->debugf('[%s] Reenter scope at scope %d', whoami(__PACKAGE__), $scope);
+      $log->debugf('[%s] Reenter scope at scope %d', whoami(__PACKAGE__), $self->{_nscope});
   }
   $self->parseDelay(0);
 
@@ -133,11 +138,11 @@ sub condExitScope {
 sub doExitScope {
   my ($self) = @_;
 
-  my $scope = $self->parseScopeLevel;
   if ($log->is_debug) {
-      $log->debugf('[%s] Removing scope %d', whoami(__PACKAGE__), $scope);
+      $log->debugf('[%s] Removing scope %d', whoami(__PACKAGE__), $self->{_nscope});
   }
   pop(@{$self->{_typedefPerScope}});
+  $self->{_nscope}--;
 
   if (@{$self->{_exitScopeCallback}}) {
       my ($ref, @args) = @{$self->{_exitScopeCallback}};
@@ -152,11 +157,10 @@ sub parseEnterTypedef {
 
   $data //= 1;
 
-  my $scope = $self->parseScopeLevel;
-  $self->{_typedefPerScope}->[$scope]->{$token} = $data;
+  $self->{_typedefPerScope}->[$self->{_nscope}]->{$token} = $data;
 
   if ($log->is_debug) {
-      $log->debugf('[%s] "%s" typedef entered at scope %d', whoami(__PACKAGE__), $token, $scope);
+      $log->debugf('[%s] "%s" typedef entered at scope %d', whoami(__PACKAGE__), $token, $self->{_nscope});
   }
 }
 
@@ -167,9 +171,8 @@ sub parseEnterEnum {
   $data //= 1;
 
   $self->{_enumAnyScope}->{$token} = $data;
-  my $scope = $self->parseScopeLevel;
   if ($log->is_debug) {
-      $log->debugf('[%s] "%s" enum entered at scope %d', whoami(__PACKAGE__), $token, $scope);
+      $log->debugf('[%s] "%s" enum entered at scope %d', whoami(__PACKAGE__), $token, $self->{_nscope});
   }
   #
   # Enum wins from now on and forever
@@ -183,7 +186,7 @@ sub parseEnterEnum {
 sub parseObscureTypedef {
   my ($self, $token, $scope) = @_;
 
-  $scope //= $self->parseScopeLevel;
+  $scope //= $self->{_nscope};
   $self->{_typedefPerScope}->[$scope]->{$token} = undef;
 
   if ($log->is_debug) {
@@ -195,7 +198,7 @@ sub parseObscureTypedef {
 sub parseIsTypedef {
   my ($self, $token) = @_;
 
-  my $scope = $self->parseScopeLevel;
+  my $scope = $self->{_nscope};
   my $rc = (exists($self->{_typedefPerScope}->[$scope]->{$token}) && defined($self->{_typedefPerScope}->[$scope]->{$token})) ? 1 : 0;
 
   if ($log->is_debug) {
@@ -211,9 +214,8 @@ sub parseIsEnum {
 
   my $rc = (exists($self->{_enumAnyScope}->{$token}) && $self->{_enumAnyScope}->{$token}) ? 1 : 0;
 
-  my $scope = $self->parseScopeLevel;
   if ($log->is_debug) {
-      $log->debugf('[%s] "%s" is an enum at scope %d? %s', whoami(__PACKAGE__), $token, $scope, $rc ? 'yes' : 'no');
+      $log->debugf('[%s] "%s" is an enum at scope %d? %s', whoami(__PACKAGE__), $token, $self->{_nscope}, $rc ? 'yes' : 'no');
   }
 
   return($rc);
@@ -233,7 +235,7 @@ MarpaX::Languages::C::AST::Scope - Scope management when translating a C source
 
 =head1 VERSION
 
-version 0.20
+version 0.27
 
 =head1 SYNOPSIS
 
